import ij.*;
import ij.process.*;
import ij.plugin.*;
import ij.plugin.filter.Analyzer;
import ij.measure.*;

/**    Counts the thresholded voxels in a stack and displays the count, average count per slice and the volume fraction.
*/
public class Voxel_Counter implements PlugIn {

	public void run(String arg) {
		ImagePlus imp = WindowManager.getCurrentImage();
		if (imp==null) {
			IJ.noImage();
			return;
		}
		if (!(imp.getProcessor() instanceof ByteProcessor)) {
			IJ.showMessage("Voxel Counter", "8-bit image or stack required.");
			return;
		}
		if (imp.getProcessor().getMinThreshold()==ImageProcessor.NO_THRESHOLD && !binaryImage(imp)) {
			IJ.showMessage("Voxel Counter", "Thresolded or binary stack required.");
			return;
		}
		IJ.run("Clear Results");
		IJ.run("Set Measurements...", "area limit decimal=2");
		Calibration cal = imp.getCalibration();
		double pw = cal.pixelWidth,  ph = cal.pixelHeight;
		cal.pixelWidth = 1.0; cal.pixelHeight = 1.0;
		int nslices = imp.getStackSize();
		for (int i=1; i<=nslices; i++) {
			imp.setSlice(i);
			IJ.run("Measure");
		}
		cal.pixelWidth = pw; cal.pixelHeight = ph;
		ResultsTable rt = Analyzer.getResultsTable();
		double sum = 0;
		for (int i=0; i<rt.getCounter(); i++)
			sum += rt.getValue(ResultsTable.AREA, i);
		IJ.write("");
		IJ.write("Voxel count: "+(int)sum);
		IJ.write("Average per slice:  "+IJ.d2s(sum/nslices,2));
		IJ.write("Volume fraction:  "+IJ.d2s((sum*100)/(imp.getWidth()*imp.getHeight()*nslices),2)+"%");
	}

	boolean binaryImage(ImagePlus imp) {
		ImageStatistics stats = imp.getStatistics();
		boolean isBinary = stats.histogram[0]+stats.histogram[255]==stats.pixelCount;
		if (isBinary) {
			boolean invertedLut = imp.isInvertedLut();
			ImageProcessor ip = imp.getProcessor();
			if (invertedLut)
				ip.setThreshold(255, 255, ImageProcessor.RED_LUT);
			else
				ip.setThreshold(0, 0, ImageProcessor.RED_LUT);
		}
		return isBinary;
	}

}
